#include "misc.h"

/* coordinates for the cube */

float coords[8][3] = {
    { -1.0, -1.0, -1.0 },
    {  1.0, -1.0, -1.0 },
    { -1.0,  1.0, -1.0 },
    {  1.0,  1.0, -1.0 },
    { -1.0, -1.0,  1.0 },
    {  1.0, -1.0,  1.0 },
    { -1.0,  1.0,  1.0 },
    {  1.0,  1.0,  1.0 },
};

/* transformed coordinates */

float trans_coords[8][4];

void draw_face(float *p1, float *p2, float *p3, float *p4)
{
    vertex_ot_t vert;
    
    vert.r = vert.g = vert.b = vert.a = 1.0f;
    vert.oa = vert.or = vert.ob = 0.0f;
    vert.flags = TA_VERTEX_NORMAL;
    
    vert.x = p1[0];
    vert.y = p1[1];
    vert.z = p1[3];
    vert.u = 0.0;
    vert.v = 0.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = p2[0];
    vert.y = p2[1];
    vert.z = p2[3];
    vert.u = 1.0;
    vert.v = 0.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = p3[0];
    vert.y = p3[1];
    vert.z = p3[3];
    vert.u = 0.0;
    vert.v = 1.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = p4[0];
    vert.y = p4[1];
    vert.z = p4[3];
    vert.u = 1.0;
    vert.v = 1.0;
    vert.flags = TA_VERTEX_EOL;
    ta->commit_vertex(&vert, sizeof(vert));
}

void cube_one_frame(float x, float y, float z, int rot)
{
    poly_hdr_t poly;
    
    /* draw cube */
 
    clear_matrix();
    apply_matrix(&screenview_matrix);
    apply_matrix(&projection_matrix);
    translate(x, y, z);

    rotate_x(rot);
    rotate_y(rot*2);
    rotate_z(rot*4);
    
    transform_coords(coords, trans_coords, 8);
    
    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565, 64, 64, inter_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);
    
    draw_face(trans_coords[4], trans_coords[5], trans_coords[0],
              trans_coords[1]);

    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565, 64, 64, fire_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);
    
    draw_face(trans_coords[4], trans_coords[0], trans_coords[6],
              trans_coords[2]);

    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565, 64, 64, stars_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);

    draw_face(trans_coords[1], trans_coords[5], trans_coords[3],
              trans_coords[7]);

    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565, 64, 64, special_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);

    draw_face(trans_coords[0], trans_coords[1], trans_coords[2],
              trans_coords[3]);
    draw_face(trans_coords[5], trans_coords[4], trans_coords[7],
              trans_coords[6]);

    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565, 64, 64, plasma_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);

    draw_face(trans_coords[2], trans_coords[3], trans_coords[6],
              trans_coords[7]);
}

void npm_cube_one_frame(float x, float y, float z, int rot, float s)
{
    poly_hdr_t poly;
    
    clear_matrix();
    apply_matrix(&screenview_matrix);
    apply_matrix(&projection_matrix);
    translate(x, y, z);

    rotate_x(rot);
    rotate_y(rot*2);
    rotate_z(rot*4);

    scale(s, s, s);
    
    transform_coords(coords, trans_coords, 8);
    
    ta->poly_hdr_txr(&poly, TA_OPAQUE, TA_RGB565_TWID, 64, 64, zoom64, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);
    
    draw_face(trans_coords[4], trans_coords[5], trans_coords[0],
              trans_coords[1]);
    
    draw_face(trans_coords[4], trans_coords[0], trans_coords[6],
              trans_coords[2]);

    draw_face(trans_coords[1], trans_coords[5], trans_coords[3],
              trans_coords[7]);

    draw_face(trans_coords[0], trans_coords[1], trans_coords[2],
              trans_coords[3]);
    draw_face(trans_coords[5], trans_coords[4], trans_coords[7],
              trans_coords[6]);

    draw_face(trans_coords[2], trans_coords[3], trans_coords[6],
              trans_coords[7]);
}
